#from pdf_parser import get_textelements_from_pdf, generate_template, get_contents_from_textelements
#import json

from pdf_to_image import get_pdf_as_image
from image_to_text import generate_text_from_image
from text_model import Page
import os
import json

class Match():

    def __init__(self, p_word="", p_page_number=0, p_block_number=0, p_line_number=0, p_word_number=0):
        self.word = p_word
        self.page_number = p_page_number
        self.block_number = p_block_number
        self.line_number = p_line_number
        self.word_number = p_word_number

    def validate_match(self, p_data):
        page = p_data[self.page_number]
        block = page.get_block_at(self.block_number)
        line = block.get_line_at(self.line_number)
        word = line.get_word_at(self.word_number)
        print(str(word) + "<->" + str(self.word))

    def __str__(self):
        return self.word + ": " + str(self.page_number) + " > " + str(self.block_number) + " > " + str(self.line_number) + " > " + str(self.word_number)

def is_template_fitting(p_template, p_page_list):
    hook = p_template[0]
    wanted_page, wanted_block, wanted_line, wanted_word, hook_word = hook
    page = p_page_list[wanted_page]
    block = page.get_block_at(wanted_block)
    line = block.get_line_at(wanted_line)
    word = line.get_word_at(wanted_word)
    return str(word) == hook_word

def save_page_list_to_file(p_page_list, p_name):
    json_list = []
    for page in p_page_list:
        json_list.append(page.to_json())
    file = open(p_name + ".ptt", "w")
    json.dump(json_list, file)
    file.close()

def get_pdf_as_text(p_path, p_seperator="/"):
    pdf_name = p_path.split(p_seperator)[-1]
    page_list = []
    if os.path.isfile(pdf_name + ".ptt"):
        json_list = json.load(open(pdf_name + ".ptt"))
        for json_page in json_list:
            page = Page.from_json(json_page)
            page_list.append(page)
    else:
        image_list = get_pdf_as_image(p_path)
        for i in range(len(image_list)):
            image = image_list[i]
            page = generate_text_from_image(image)
            page.number = i
            page_list.append(page)
        save_page_list_to_file(page_list, pdf_name)
    return page_list

#def get_text_from_pdf(p_path):
#    pdf_name = p_path.split("/")[-1:]
#    if os.path.isfile(p_path):


def evaluate_pdf(p_path, p_template):
    page_list = get_pdf_as_text(p_path)

    if is_template_fitting(p_template, page_list):
        print("Template fits")
    else:
        print("Temṕlate not suitable for this pdf")
        return

    for entry in p_template[1].items():
        key, offset_set = entry
        for page in page_list:
            for block in page.get_block_list():
                for line in block.get_line_list():
                    for word in line.get_word_list():
                        if str(word) == key:
                            wanted_block = block.index - offset_set[1]
                            wanted_line = line.index - offset_set[2]
                            wanted_word = word.index - offset_set[3]

                            print(key)

                            value_block = page.get_block_at(wanted_block)
                            value_line = value_block.get_line_at(wanted_line)
                            value_word = value_line.get_word_at(wanted_word)
                            print(value_word)

    

def create_template(p_name, p_path, p_example, p_hook):
    page_list = get_pdf_as_text(p_path)

    matches_dict = {}
    for key in p_example:
        matches_dict[key] = [None, None]
    hook = None
    template = {}
    for page in page_list:
            for block in page.get_block_list():
                for line in block.get_line_list():
                    for word in line.get_word_list():
                        for example in p_example.items():
                            key, value = example
                            if str(word) == key:
                                key_match = Match(key)
                                key_match.page_number = page.index
                                key_match.block_number = block.index
                                key_match.line_number = line.index
                                key_match.word_number = word.index
                                match_pair = matches_dict[key]
                                match_pair[0] = key_match
                                matches_dict[key] = match_pair
                            elif str(word) == value:
                                value_match = Match(value)
                                value_match.page_number = page.index
                                value_match.block_number = block.index
                                value_match.line_number = line.index
                                value_match.word_number = word.index
                                match_pair = matches_dict[key]
                                match_pair[1] = value_match
                                matches_dict[key] = match_pair
                            elif str(word) == p_hook:
                                hook = [page.index, block.index, line.index, word.index, p_hook]

    for match_pair in matches_dict.values():
        key_match, value_match = match_pair
        if key_match is not None:
            key_match.validate_match(page_list)
        if value_match is not None:
            value_match.validate_match(page_list)
        if key_match is None or value_match is None:
            continue
        page_offset = key_match.page_number - value_match.page_number
       # block_offset = key_match.block_number - value_match.block_number
       # line_offset = key_match.line_number - value_match.line_number
       # word_offset = key_match.word_number - value_match.word_number

       # offset_set = [page_offset, block_offset, line_offset, word_offset]
        key = key_match.word
        value_position = [page_offset, value_match.block_number, value_match.line_number, value_match.word_number]
        template[key] = value_position

    return (hook, template)
